#! /bin/sh
##### DO NOT ADD LINE BEFORE THIS ONE: --install depends on it

VERSION=0.12
if ( echo $@ | grep -e "--version" ) 1> /dev/null; then
    echo "Installation script:  $0"
    echo "acpi script (Version: $VERSION)"
    exit 0
fi

# acpi - a little script to deal with events on ACPI systems
# Changelog
# 0.12 : - tmp dir test output redirected to /dev/null
# 0.9 : - change suspend script to hibernate
# 0.4 : - align version with swsusp.sh
# 0.1 : - initial version

PATH=/bin:/sbin:/usr/bin:/usr/bin/X11
TMP=/dev/shm/acpi.$$
if ! touch $TMP 2> /dev/null ;then
    TMP=/tmp/acpi.$$
    if ! touch $TMP 2> /dev/null ;then
	echo "Can't open a temporary file in /dev/shm nor /tmp"
	echo "Aborting !"
	exit 1
    fi
fi

ID=`id -u`
if [ "$ID" != "0" ]; then
    echo "You must be root to use this script."
    echo "Use sudo to enable it for users."
    exit 1
fi

if ( echo $@ | grep -e "--silent" ) 1> /dev/null; then
    VERBOSE='/dev/null'
else
    if [ -w /dev/tty ] ;then
	VERBOSE='/dev/tty'
    else
	VERBOSE='/dev/console'
    fi
fi

if ( echo $@ | grep -e "--force" ) 1> /dev/null; then
    FORCE='yes'
else
    FORCE='no'
fi

######################################### INSTALL #####################################################
BINNAME=acpish
if ( echo $@ | grep -e "--install" ) 1> /dev/null; then    
    DISTRIB=Unknown
    if [ -d /etc/rc.config.d ]
    then
	DISTRIB=SuSE
    elif [ -x /usr/X11R6/bin/drakconf -o -x /usr/sbin/drakconf ]
    then
	DISTRIB=Mandrake
    fi

    case "$DISTRIB" in
	SuSE)
	    CONFIGFILE=/etc/rc.config.d/acpish.rc.config
	    INSTALLDIR=/usr/sbin
	    SHELL=/bin/bash
	    LOCKFILE=/var/lock/subsys/acpish
	;;
	Mandrake)
	    CONFIGFILE=/etc/acpish.conf
	    INSTALLDIR=/usr/local/sbin
	    SHELL=/bin/bash
	    LOCKFILE=/var/lock/subsys/acpish
	;;
	*) 
	    CONFIGFILE=/etc/acpish.conf
	    INSTALLDIR=/usr/local/sbin
	    SHELL=/bin/bash
	    LOCKFILE=/var/lock/acpish
	;;
    esac
    SUSPEND=$INSTALLDIR/hibernate
    HALT="`which shutdown`"
    
    MODE=755
    DATE=`date`
    cat > $TMP <<EOF
#! $SHELL

if ( echo \$@ | grep -e "--version" ) 1> /dev/null; then
    echo "Script:       $0"
    echo "Installation: $DATE"
    echo "Distribution: $DISTRIB"
    echo "Version:      $VERSION"
    exit 0
fi

CONFIGFILE=$CONFIGFILE
LOCKFILE=$LOCKFILE
# DO NOT EDIT THIS FILE (change the above script and reinstall)
EOF

    for file in /proc/acpi/event ;do
	if [ -f $file ] ;then
	    echo "ACPI_EVENT_PROC=\"$file\"" >> $TMP
	fi
    done

    nb=`wc -l < $0`
    nb=$[ $nb - 2 ]
    tail -$nb $0 >> $TMP

    if [ ! -d $INSTALLDIR ] ;then
	echo "Directory $INSTALLDIR doesn't exist: installation aborted"
	rm $TMP
	exit 2
    fi
   
    echo "Creating $INSTALLDIR/$BINNAME" > $VERBOSE
    if ! (mv -f $TMP $INSTALLDIR/$BINNAME) ;then
	echo "Cannot install $INSTALLDIR/$BINNAME"
	rm $TMP
	exit 1
    fi
    chmod $MODE $INSTALLDIR/$BINNAME

    cat > $TMP~ <<EOF
#-*-mode: sh-*-
# Configuration of acpi script
#

# Power button action
# Default: "bash -c \"if ! xmessage -font huge -timeout 30 -buttons Cancel:0,Halt:1 -default Cancel -geometry +500+380 'Halt computer ?';then $HALT -h -k now 'Power button'; sleep 10; $HALT -h -k -t 5 now 'Halting machine'; fi\" &"
POWER_BUTTON_ACTION="bash -c \"if ! xmessage -font huge -timeout 30 -buttons Cancel:0,Halt:1 -default Cancel -geometry +500+380 'Halt computer ?';then $HALT -h -k now 'Power button'; sleep 10; $HALT -h -t 5 now 'Halting machine'; elif ! xmessage -font huge -timeout 30 -buttons Cancel:0,Stop:1 -default Cancel -geometry +500+380 'Stop ACPI monitoring ?';then kill \$\$; fi\" &"

# Lid closing action
# Default: "$SUSPEND"
LID_ACTION="$SUSPEND"
EOF
    if [ ! -f $CONFIGFILE -o "$FORCE" = "yes" ] ;then
	echo "Cration du fichier $CONFIGFILE" > $VERBOSE
	mv $TMP~ $CONFIGFILE 
    else
	echo "Configuration file $CONFIGFILE exists: not overwritten" > $VERBOSE
	diff -u $CONFIGFILE $TMP~ > $VERBOSE
	rm $TMP~
    fi

    cat > $TMP <<EOF
# This file is part of $BINNAME script
# Installation: $DATE
# Script:       $0
# Distro:       $DISTRIB
# DO NOT EDIT THIS FILE (change the above script and reinstall)
{
    if(substr(\$1,1,7) == "button/") {
	printf("Button action: ");
	if(\$2 == "PWRF") {
	    printf("Power");
	    system(poweraction);
	} else {
	    if (\$2 == "LID") {
		printf("Lid");
		system(lidaction);
	    }
	}
	printf("\n");
    }
}
EOF
    echo "Creating $INSTALLDIR/$BINNAME.awk" > $VERBOSE
    if ! (mv -f $TMP $INSTALLDIR/$BINNAME.awk) ;then
	echo "Cannot install $INSTALLDIR/$BINNAME.awk"
	rm $TMP
	exit 1
    fi
    chmod $MODE $INSTALLDIR/$BINNAME.awk
    chmod a-x $INSTALLDIR/$BINNAME.awk
    exit 0
fi


############################### END OF INSTALL ###################################################

EXE=`basename $0`
cat > $TMP <<EOF
The script file $EXE uses a configuration file $CONFIGFILE.
It is intended to monitor ACPI events such as power button pressing or lid
closure. Its options are as follows:
--help		Prints this help.
--install	Install this file in $INSTALLDIR and a configuration
		template as $CONFIGFILE. Existing script will be 
		overwritten, but not configuration file, unles --force
		is used. 
--force		Overwrite existing configuration file in installation
		procedure. 
--silent	Be silent on processing (for using as a background system process).
EOF

if [ -f "$CONFIGFILE" ]
then
    . $CONFIGFILE
else
    echo "No configuration file $CONFIGFILE !!!"
    cat $TMP
    rm $TMP
    exit 1
fi

if ( echo $@ | grep -e "--help" ) 1> /dev/null; then
    cat $TMP
    rm $TMP
    exit 0
fi
rm $TMP

if [ -f "$LOCKFILE" ] ;then
    pid=`cat $LOCKFILE`
    if (ps $pid > /dev/null 2> /dev/null) ;then
	echo "File $LOCKFILE is present. Monitoring script `cat $LOCKFILE`"
	echo "should be in progress. If this is not the case, then erase"
	echo "this file"
	exit 1
    fi
fi
echo $$ > $LOCKFILE

while /bin/true
do
    awk --assign poweraction="$POWER_BUTTON_ACTION" --assign lidaction="$LID_ACTION" -f $0.awk < $ACPI_EVENT_PROC
    # on suspend, the ACPI event file can close leading to awk exit
    sleep 10 
done
