#! /bin/sh
##### DO NOT ADD LINE BEFORE THIS ONE: --install depends on it

VERSION=0.12
if ( echo $@ | grep -e "--version" ) 1> /dev/null; then
    echo "Installation script:     $0"
    echo "battery script (Version: $VERSION)"
    exit 0
fi

# battery - a little script to check the power on ACPI systems
# Changelog
# 0.12 : - add /proc/acpi/ac_adpater/ACAD/state for checking AC status (mrl21_pub@netcourrier.com)
#        - tmp dir test output redirected to /dev/null
#        - add a test to check correct reading of acpi proc files
#        - no >$VERBOSE in installation part
#        - Create a battery.conf.new or .battery.new file to help merging between versions of this script.
# 0.11 : - add a test to check whether action variables are empty 
# 0.9 : - change suspend script to hibernate
# 0.4 : - align version with swsusp.sh
# 0.2 : - add percentage during charging
#       - use acpi interface to reduce cpu speed when on battery power
#       - use hdparm to timeout disk when on battery power
#       - start/stop scripts when AC disconnected
#       - Modify Mandrake detection since drakconf place has changed.
#       - /dev/tty may not be available: adding a test and /dev/console output
#       - Modify percentage parameter passation to command script


PATH=/bin:/sbin:/usr/bin:/usr/bin/X11
TMP=/dev/shm/battery.$$
if ! touch $TMP 2> /dev/null ;then
    TMP=/tmp/battery.$$
    if ! touch $TMP 2> /dev/null ;then
	echo "Can't open a temporary file in /dev/shm nor /tmp"
	echo "Aborting !"
	exit 1
    fi
fi

if ( echo $@ | grep -e "--silent" ) 1> /dev/null; then
    VERBOSE='/dev/null'
else
    if [ -w /dev/tty ] ;then
	VERBOSE='/dev/tty'
    else
	VERBOSE='/dev/console'
    fi
fi

if ( echo $@ | grep -e "--force" ) 1> /dev/null; then
    FORCE='yes'
else
    FORCE='no'
fi

######################################### INSTALL #####################################################
BINNAME=battery
if ( echo $@ | grep -e "--install" ) 1> /dev/null; then

    
    DISTRIB=Unknown
    if [ -d /etc/rc.config.d ]
    then
	DISTRIB=SuSE
    elif [ -x /usr/X11R6/bin/drakconf -o -x /usr/sbin/drakconf ]
    then
	DISTRIB=Mandrake
    fi

    case "$DISTRIB" in
	SuSE)
	    CONFIGFILE=/etc/rc.config.d/battery.rc.config
	    INSTALLDIR=/usr/sbin
	    SHELL=/bin/bash
	    LOCKFILE=/var/lock/subsys/battery
	    DEFAULTSSERVICES="cron atd"
	;;
	Mandrake)
	    CONFIGFILE=/etc/battery.conf
	    INSTALLDIR=/usr/local/sbin
	    SHELL=/bin/bash
	    LOCKFILE=/var/lock/subsys/battery
	    DEFAULTSSERVICES="anacron atd crond"
	;;
	*) 
	    CONFIGFILE=/etc/battery.conf
	    INSTALLDIR=/usr/local/sbin
	    SHELL=/bin/bash
	    LOCKFILE=/var/lock/battery
	    DEFAULTSSERVICES="crond atd"
	;;
    esac
    SUSPEND=$INSTALLDIR/hibernate

    SERVICES=
    for service in $DEFAULTSSERVICES ;do
	if [ -x /etc/init.d/$service ]
	then
	    SERVICES="$SERVICES $service"
	fi
    done
    MODE=755
    ID=`id -u`
    if [ "$ID" != "0" ]; then
	SUSPEND="sudo $INSTALLDIR/hibernate"
	CONFIGFILE=$HOME/.battery
	LOCKFILE=$HOME/.battery.lck
	if [ -f $CONFIGFILE ] ;then
	    . $CONFIGFILE
	    cat > $TMP~ <<EOF
#-*-mode: sh-*-
# Configuration of battery script
#

# This variable is for user mode installation.
INSTALLDIR=$INSTALLDIR
EOF
	else
	    cat > $CONFIGFILE <<EOF
#-*-mode: sh-*-
# Configuration of battery script
#

# This variable is for user mode installation.
INSTALLDIR=/tmp
EOF
	    echo "Edit $CONFIGFILE to set INSTALLDIR to something you like and rerun $BINNAME.sh --install"
	    exit 1
	fi
	echo -n > $TMP~~
    else
	cat > $TMP~ <<EOF
#-*-mode: sh-*-
# Configuration of battery script
#
EOF
	cat > $TMP~~ <<EOF

# Disk timeout rates when AC, Battery powered
# see man hdparm for more details (option -S)
# Default is no spin down and 30 seconds
# Default: 0,6
DISK_TIMEOUT_SETTINGS=0,6

# Disk advanced power management settings when AC, Battery powered
# see man hdparm for more details (option -B)
# Default is no spin down and apm level 1 (no se !)
# Default: 255,1
DISK_APM_SETTINGS=255,1

# Device to manage with hdparm using the two above settings
# Set to none to disable.
# Default: /dev/hda
DISK_DEVICE=/dev/hda

# Services to stop/start when AC is disconnected/connected
# Default: $SERVICES
POWERFULL_SERVICES="$SERVICES"
EOF
    fi

    DATE=`date`
    cat > $TMP <<EOF
#! $SHELL

if ( echo \$@ | grep -e "--version" ) 1> /dev/null; then
    echo "Script:       $0"
    echo "Installation: $DATE"
    echo "Distribution: $DISTRIB"
    echo "Version:      $VERSION"
    exit 0
fi

CONFIGFILE=$CONFIGFILE
LOCKFILE=$LOCKFILE
# DO NOT EDIT THIS FILE (change the above script and reinstall)
EOF

    for file in /proc/acpi/ac_adapter/0/status /proc/acpi/ac_adapter/ADP0/state /proc/acpi/ac_adpater/ACAD/state ;do
	if [ -f $file ] ;then
	    echo "ACPI_AC_PROC=\"$file\"" >> $TMP
	fi
    done
    for file in /proc/acpi/battery/0/status /proc/acpi/battery/BAT0/state ;do
	if [ -f $file ] ;then
	    echo "ACPI_BATT_STATUS_PROC=\"$file\"" >> $TMP
	fi
    done
    for file in /proc/acpi/battery/0/info /proc/acpi/battery/BAT0/info ;do
	if [ -f $file ] ;then
	    echo "ACPI_BATT_INFO_PROC=\"$file\"" >> $TMP
	fi
    done
    for file in /proc/acpi/processor/CPU0/performance ;do
	if [ -f $file ] ;then
	    echo "ACPI_CPU_PERF_PROC=\"$file\"" >> $TMP
	fi
    done
    nb=`wc -l < $0`
    nb=$[ $nb - 2 ]
    tail -$nb $0 >> $TMP

    if [ ! -d $INSTALLDIR ] ;then
	echo "Directory $INSTALLDIR doesn't exist: installation aborted"
	rm $TMP $TMP~
	exit 2
    fi
   
    echo "Creating $INSTALLDIR/$BINNAME"
    if ! (mv -f $TMP $INSTALLDIR/$BINNAME) ;then
	echo "Cannot install $INSTALLDIR/$BINNAME"
	rm $TMP $TMP~
	exit 1
    fi
    chmod $MODE $INSTALLDIR/$BINNAME

    cat >> $TMP~ <<EOF

# Percentages of battery capacity that cause low,warning,critical
# notification
# Default: 50,10,5
BATTERY_LEVELS=50,10,5

# Warning action
# Default: "bash -c \"if ! xmessage -font huge -timeout 30 -buttons OK:0,Suspend:1 -default OK -geometry +500+380 'Battery is at \\\$1 %';then $SUSPEND; fi\" &"
BATTERY_WARNING_ACTION="bash -c \"if ! xmessage -font huge -timeout 30 -buttons OK:0,Suspend:1 -default OK -geometry +500+380 'Battery is at \\\$1 %';then $SUSPEND; fi\" &"

# Critical action
# Default: "$SUSPEND --force --kill"
BATTERY_CRITICAL_ACTION="$SUSPEND --force --kill"

# Refresh rates in seconds during AC, NORMAL, LOW, WARNING, CRITICAL states
# Default: 20,180,120,60,30
BATTERY_REFRESH_RATES=20,180,120,60,30
EOF
    cat $TMP~~ >> $TMP~
    rm $TMP~~
    
    if [ ! -f $CONFIGFILE -o "$FORCE" = "yes" ] ;then
	echo "Cration du fichier $CONFIGFILE"
	mv $TMP~ $CONFIGFILE 
    else
	echo "Configuration file $CONFIGFILE exists: not overwritten"
	echo -e "These are the differences:\n" 
	diff -u $CONFIGFILE $TMP~
	echo "Creating file $CONFIGFILE.new"
	mv $TMP~ $CONFIGFILE.new
    fi

    exit 0
fi


############################### END OF INSTALL ###################################################

EXE=`basename $0`
cat > $TMP <<EOF
The script file $EXE uses a configuration file $CONFIGFILE.
It is intended to monitor battery capcaity on an ACPI system. Its 
options are as follows:
--help		Prints this help.
--install	Install this file in $INSTALLDIR and a configuration
		template as $CONFIGFILE. Existing script will be 
		overwritten, but not configuration file, unles --force
		is used. 
--force		Overwrite existing configuration file in installation
		procedure. 
--xterm		Launch the script in its proper xterm window.
--silent	Be silent on processing (for using as a background system process).
EOF

if [ -f "$CONFIGFILE" ]
then
    . $CONFIGFILE
else
    echo "No configuration file $CONFIGFILE !!!"
    cat $TMP
    rm $TMP
    exit 1
fi

if ( echo $@ | grep -e "--help" ) 1> /dev/null; then
    cat $TMP
    rm $TMP
    exit 0
fi
rm $TMP

if ( echo $@ | grep -e "--xterm" ) 1> /dev/null; then
    exec xterm -name $BINNAME -bg black -fg black -sl 0 +sb -geometry 6x1 -e $INSTALLDIR/$BINNAME
fi

if [ -f "$LOCKFILE" ] ;then
    pid=`cat $LOCKFILE`
    if (ps $pid > /dev/null 2> /dev/null) ;then
	echo "File $LOCKFILE is present. Monitoring script `cat $LOCKFILE`"
	echo "should be in progress. If this is not the case, then erase"
	echo "this file"
	exit 1
    fi
fi
echo $$ > $LOCKFILE

AC_RATE=`echo $BATTERY_REFRESH_RATES | cut -f1 -d,`
NORMAL_RATE=`echo $BATTERY_REFRESH_RATES | cut -f2 -d,`
LOW_RATE=`echo $BATTERY_REFRESH_RATES | cut -f3 -d,`
WARNING_RATE=`echo $BATTERY_REFRESH_RATES | cut -f4 -d,`
CRITICAL_RATE=`echo $BATTERY_REFRESH_RATES | cut -f5 -d,`

LOW_LEVEL=`echo $BATTERY_LEVELS | cut -f1 -d,`
WARNING_LEVEL=`echo $BATTERY_LEVELS | cut -f2 -d,`
CRITICAL_LEVEL=`echo $BATTERY_LEVELS | cut -f3 -d,`

AC_DISK_TIMEOUT=`echo $DISK_TIMEOUT_SETTINGS | cut -f1 -d,`
BATT_DISK_TIMEOUT=`echo $DISK_TIMEOUT_SETTINGS | cut -f2 -d,`

AC_DISK_APM=`echo $DISK_APM_SETTINGS | cut -f1 -d,`
BATT_DISK_APM=`echo $DISK_APM_SETTINGS | cut -f2 -d,`

SERVICES_STATUS=
while true
do
  max=`grep -i "last full capacity" $ACPI_BATT_INFO_PROC | cut -f9 -d' '`
  cur=`grep -i "remaining capacity" $ACPI_BATT_STATUS_PROC | cut -f8 -d' '`

  if [ "$max" = "" -o "$cur" = "" ]
  then
      echo "Error reading $ACPI_BATT_INFO_PROC ($max) or $ACPI_BATT_STATUS_PROC ($cur)"
      exit 1
  fi

  perc=$[ $cur * 100 ]
  perc=$[ $perc / $max ]
  cpustate=`grep -i "active state" $ACPI_CPU_PERF_PROC | cut -f14 -d' '`
  if grep -iq on-line $ACPI_AC_PROC ;then
      if [ "$cpustate" != "P0" -a -w $ACPI_CPU_PERF_PROC ]
      then
	  echo 0 > $ACPI_CPU_PERF_PROC
      fi
      test=`hdparm -C $DISK_DEVICE | grep standby | wc -l`
      if [ $test -ge 1 ] ;then
	  hdparm -S $AC_DISK_TIMEOUT $DISK_DEVICE
	  hdparm -B $AC_DISK_APM $DISK_DEVICE
	  if [ "$SERVICES_STATUS" != "start" ] ;then
	      SERVICES_STATUS=start
	      for service in $POWERFULL_SERVICES ;do
		  /etc/init.d/$service $SERVICES_STATUS
	      done
	  fi
      fi
      
      charging=`grep -i "charging state" $ACPI_BATT_STATUS_PROC | cut -f12 -d' '`
      if [ "$charging" = "charging" ];then      
	  echo -en "\033[1;34m" > $VERBOSE
	  echo -n "CA$perc%" > $VERBOSE
      else
	  echo -en "\033[1;36m" > $VERBOSE
	  echo -n "  AC" > $VERBOSE
      fi
      sleep=$AC_RATE
  else
      if [ "$cpustate" != "P1" -a -w $ACPI_CPU_PERF_PROC ]
      then
	  echo 1 > $ACPI_CPU_PERF_PROC
      fi
      test=`hdparm -C $DISK_DEVICE | grep active | wc -l`
      if [ $test -ge 1 ] ;then
	  hdparm -S $BATT_DISK_TIMEOUT $DISK_DEVICE
	  hdparm -B $BATT_DISK_APM $DISK_DEVICE
	  
	  if [ "$SERVICES_STATUS" != "stop" ] ;then
	      SERVICES_STATUS=stop
	      for service in $POWERFULL_SERVICES ;do
		  /etc/init.d/$service $SERVICES_STATUS
	      done
	  fi
      fi

      if [ $perc -lt $CRITICAL_LEVEL -a "$BATTERY_CRITICAL_ACTION" != "" ] ;then
	  echo $BATTERY_CRITICAL_ACTION > $TMP
	  chmod 700 $TMP
	  bash -c "$TMP $perc"
	  echo -en "\033[1;35m" > $VERBOSE
	  sleep=$CRITICAL_RATE
      elif [ $perc -le $WARNING_LEVEL -a "$BATTERY_WARNING_ACTION" != "" ] ;then
	  echo $BATTERY_WARNING_ACTION > $TMP
	  chmod 700 $TMP
	  bash -c "$TMP $perc" &	  
	  echo -en "\033[1;31m" > $VERBOSE
	  sleep=$WARNING_RATE
      else
	  if [ $perc -le $LOW_LEVEL ] ;then
	      echo -en "\033[1;33m" > $VERBOSE
	      sleep=$LOW_RATE
	  else
	      echo -en "\033[1;32m" > $VERBOSE
	      sleep=$NORMAL_RATE
	  fi
      fi
      echo -n " $perc %" > $VERBOSE
  fi
  echo -en "\033[0;39m" > $VERBOSE
  sleep $sleep
  echo > $VERBOSE
done
